-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxiliary"
M.defaultOrder = 80

M.isActive = false
M.isActing = false

local CMU = nil
local isDebugEnabled = false

local controlParameters = {isEnabled = true}
local initialControlParameters

local configPacket = {sourceType = "adaptiveSprings", packetType = "config", config = controlParameters}
local debugPacket = {sourceType = "adaptiveSprings"}

local springHydro = nil
local springBeams = {}
local springModes = {}

local function setSpringsMode(modeName)
  local mode = springModes[modeName]
  if not mode then
    log("E", "adaptiveSprings.setSpringsMode", "Can't find mode: " .. modeName)
    return
  end
  
  for _, cid in ipairs(springBeams) do
    local beam = v.data.beams[cid]
    obj:setBeamSpringDamp(cid, beam.beamSpring * mode.beamSpringCoef, -1, beam.beamDamp, -1)
  end

  electrics.values[springHydro] = mode.springLengthCoef
end

local function init(jbeamData)
  springHydro = jbeamData.springHydro or "springHydro"

  local springBeamNames = jbeamData.springBeamNames or {}
  for _, b in pairs(v.data.beams) do
    if b.name then
      for _, name in pairs(springBeamNames) do
        if b.name == name then
          table.insert(springBeams, b.cid)
        end
      end
    end
  end
  for _, mode in pairs(tableFromHeaderTable(jbeamData.modes or {})) do
    springModes[mode.name] = {
      springLengthCoef = mode.springLengthCoef or 1,
	  beamSpringCoef = mode.beamSpringCoef or 1
    }
  end

  local nameString = jbeamData.name
  local slashPos = nameString:find("/", -nameString:len())
  if slashPos then
    nameString = nameString:sub(slashPos + 1)
  end
  debugPacket.sourceName = nameString

  M.isActive = true
end

local function setDebugMode(debugEnabled)
  isDebugEnabled = debugEnabled
end

local function registerCMU(cmu)
  CMU = cmu
end

local function shutdown()
  M.isActive = false
  M.updateGFX = nil
  M.update = nil
end

local function setParameters(parameters)
  if parameters.springMode then
    setSpringsMode(parameters.springMode)
  end
end

local function setConfig(configTable)
  controlParameters = configTable
end

local function getConfig()
  return deepcopy(controlParameters)
end

local function sendConfigData()
  configPacket.config = controlParameters
  CMU.sendDebugPacket(configPacket)
end

M.init = init
M.registerCMU = registerCMU
M.setDebugMode = setDebugMode
M.shutdown = shutdown
M.setParameters = setParameters
M.setConfig = setConfig
M.getConfig = getConfig
M.sendConfigData = sendConfigData

return M
